@@ -1210,7 +1210,8 @@ def downgrade(*, to_revision, from_revision=None, show_sql_only=False, session:
     config = _get_alembic_config()
     # Check if downgrade is less than 3.0.0 and requires that `ab_user` fab table is present
     if _revision_greater(config, _REVISION_HEADS_MAP["3.0.0"], to_revision):
-        if conf.getboolean("core", "unit_test_mode"):
+        unitest_mode = conf.getboolean("core", "unit_test_mode")
+        if unitest_mode:
             try:
                 from airflow.providers.fab.auth_manager.models.db import FABDBManager
 
@@ -1218,13 +1219,12 @@ def downgrade(*, to_revision, from_revision=None, show_sql_only=False, session:
                 dbm.initdb()
             except ImportError:
                 log.warning("Import error occurred while importing FABDBManager. Skipping the check.")
-                pass
-        if not inspect(settings.engine).has_table("ab_user"):
-            log.error(
+                return
+        if not inspect(settings.engine).has_table("ab_user") and not unitest_mode:
+            raise AirflowException(
                 "Downgrade to revision less than 3.0.0 requires that `ab_user` table is present. "
                 "Please add FabDBManager to [core] external_db_managers and run fab migrations before proceeding"
             )
-            return
     with create_global_lock(session=session, lock=DBLocks.MIGRATIONS):
         if show_sql_only:
             log.warning("Generating sql scripts for manual migration.")
