@@ -1,5 +1,4 @@
 # Licensed to the Apache Software Foundation (ASF) under one
-# Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
 # distributed with this work for additional information
 # regarding copyright ownership.  The ASF licenses this file
@@ -20,6 +19,7 @@ from __future__ import annotations
 
 import contextlib
 import logging
+import urllib.parse
 
 import httpx
 import pytest
@@ -148,12 +148,12 @@ class TestXComsGetEndpoint:
                 },
                 id="-4",
             ),
-            pytest.param(-3, 200, {"key": "xcom_1", "value": "f"}, id="-3"),
-            pytest.param(-2, 200, {"key": "xcom_1", "value": "o"}, id="-2"),
-            pytest.param(-1, 200, {"key": "xcom_1", "value": "b"}, id="-1"),
-            pytest.param(0, 200, {"key": "xcom_1", "value": "f"}, id="0"),
-            pytest.param(1, 200, {"key": "xcom_1", "value": "o"}, id="1"),
-            pytest.param(2, 200, {"key": "xcom_1", "value": "b"}, id="2"),
+            pytest.param(-3, 200, "f", id="-3"),
+            pytest.param(-2, 200, "o", id="-2"),
+            pytest.param(-1, 200, "b", id="-1"),
+            pytest.param(0, 200, "f", id="0"),
+            pytest.param(1, 200, "o", id="1"),
+            pytest.param(2, 200, "b", id="2"),
             pytest.param(
                 3,
                 404,
@@ -207,10 +207,72 @@ class TestXComsGetEndpoint:
             session.add(x)
         session.commit()
 
-        response = client.get(f"/execution/xcoms/dag/runid/task/xcom_1?offset={offset}")
+        response = client.get(f"/execution/xcoms/dag/runid/task/xcom_1/item/{offset}")
         assert response.status_code == expected_status
         assert response.json() == expected_json
 
+    @pytest.mark.parametrize(
+        "key",
+        [
+            pytest.param(slice(None, None, None), id=":"),
+            pytest.param(slice(None, None, -2), id="::-2"),
+            pytest.param(slice(None, 2, None), id=":2"),
+            pytest.param(slice(None, 2, -1), id=":2:-1"),
+            pytest.param(slice(None, -2, None), id=":-2"),
+            pytest.param(slice(None, -2, -1), id=":-2:-1"),
+            pytest.param(slice(1, None, None), id="1:"),
+            pytest.param(slice(2, None, -1), id="2::-1"),
+            pytest.param(slice(1, 2, None), id="1:2"),
+            pytest.param(slice(2, 1, -1), id="2:1:-1"),
+            pytest.param(slice(1, -1, None), id="1:-1"),
+            pytest.param(slice(2, -2, -1), id="2:-2:-1"),
+            pytest.param(slice(-2, None, None), id="-2:"),
+            pytest.param(slice(-1, None, -1), id="-1::-1"),
+            pytest.param(slice(-2, -1, None), id="-2:-1"),
+            pytest.param(slice(-1, -3, -1), id="-1:-3:-1"),
+        ],
+    )
+    def test_xcom_get_with_slice(self, client, dag_maker, session, key):
+        xcom_values = ["f", None, "o", "b"]
+
+        class MyOperator(EmptyOperator):
+            def __init__(self, *, x, **kwargs):
+                super().__init__(**kwargs)
+                self.x = x
+
+        with dag_maker(dag_id="dag"):
+            MyOperator.partial(task_id="task").expand(x=xcom_values)
+        dag_run = dag_maker.create_dagrun(run_id="runid")
+        tis = {ti.map_index: ti for ti in dag_run.task_instances}
+
+        for map_index, db_value in enumerate(xcom_values):
+            if db_value is None:  # We don't put None to XCom.
+                continue
+            ti = tis[map_index]
+            x = XComModel(
+                key="xcom_1",
+                value=db_value,
+                dag_run_id=ti.dag_run.id,
+                run_id=ti.run_id,
+                task_id=ti.task_id,
+                dag_id=ti.dag_id,
+                map_index=map_index,
+            )
+            session.add(x)
+        session.commit()
+
+        qs = {}
+        if key.start is not None:
+            qs["start"] = key.start
+        if key.stop is not None:
+            qs["stop"] = key.stop
+        if key.step is not None:
+            qs["step"] = key.step
+
+        response = client.get(f"/execution/xcoms/dag/runid/task/xcom_1/slice?{urllib.parse.urlencode(qs)}")
+        assert response.status_code == 200
+        assert response.json() == ["f", "o", "b"][key]
+
 
 class TestXComsSetEndpoint:
     @pytest.mark.parametrize(
