@@ -61,6 +61,7 @@ from airflow.sdk.api.datamodels._generated import (
     BundleInfo,
     ConnectionResponse,
     DagRunStateResponse,
+    InactiveAssetsResponse,
     PrevSuccessfulDagRunResponse,
     TaskInstance,
     TaskInstanceState,
@@ -74,6 +75,8 @@ from airflow.sdk.api.datamodels._generated import (
     TriggerDAGRunPayload,
     VariableResponse,
     XComResponse,
+    XComSequenceIndexResponse,
+    XComSequenceSliceResponse,
 )
 from airflow.sdk.exceptions import ErrorType
 
@@ -206,6 +209,24 @@ class AssetEventDagRunReferenceResult(AssetEventDagRunReference):
         )
 
 
+class InactiveAssetsResult(InactiveAssetsResponse):
+    """Response of InactiveAssets requests."""
+
+    type: Literal["InactiveAssetsResult"] = "InactiveAssetsResult"
+
+    @classmethod
+    def from_inactive_assets_response(
+        cls, inactive_assets_response: InactiveAssetsResponse
+    ) -> InactiveAssetsResult:
+        """
+        Get InactiveAssetsResponse from InactiveAssetsResult.
+
+        InactiveAssetsResponse is autogenerated from the API schema, so we need to convert it to InactiveAssetsResult
+        for communication between the Supervisor and the task process.
+        """
+        return cls(**inactive_assets_response.model_dump(exclude_defaults=True), type="InactiveAssetsResult")
+
+
 class XComResult(XComResponse):
     """Response to ReadXCom request."""
 
@@ -227,6 +248,24 @@ class XComCountResponse(BaseModel):
     type: Literal["XComLengthResponse"] = "XComLengthResponse"
 
 
+class XComSequenceIndexResult(BaseModel):
+    root: JsonValue
+    type: Literal["XComSequenceIndexResult"] = "XComSequenceIndexResult"
+
+    @classmethod
+    def from_response(cls, response: XComSequenceIndexResponse) -> XComSequenceIndexResult:
+        return cls(root=response.root, type="XComSequenceIndexResult")
+
+
+class XComSequenceSliceResult(BaseModel):
+    root: list[JsonValue]
+    type: Literal["XComSequenceSliceResult"] = "XComSequenceSliceResult"
+
+    @classmethod
+    def from_response(cls, response: XComSequenceSliceResponse) -> XComSequenceSliceResult:
+        return cls(root=response.root, type="XComSequenceSliceResult")
+
+
 class ConnectionResult(ConnectionResponse):
     type: Literal["ConnectionResult"] = "ConnectionResult"
 
@@ -352,8 +391,11 @@ ToTask = Annotated[
         TICount,
         TaskStatesResult,
         VariableResult,
-        XComResult,
         XComCountResponse,
+        XComResult,
+        XComSequenceIndexResult,
+        XComSequenceSliceResult,
+        InactiveAssetsResult,
         OKResponse,
     ],
     Field(discriminator="type"),
@@ -451,6 +493,17 @@ class GetXComSequenceItem(BaseModel):
     type: Literal["GetXComSequenceItem"] = "GetXComSequenceItem"
 
 
+class GetXComSequenceSlice(BaseModel):
+    key: str
+    dag_id: str
+    run_id: str
+    task_id: str
+    start: int | None
+    stop: int | None
+    step: int | None
+    type: Literal["GetXComSequenceSlice"] = "GetXComSequenceSlice"
+
+
 class SetXCom(BaseModel):
     key: str
     value: Annotated[
@@ -557,6 +610,11 @@ class GetAssetEventByAssetAlias(BaseModel):
     type: Literal["GetAssetEventByAssetAlias"] = "GetAssetEventByAssetAlias"
 
 
+class ValidateInletsAndOutlets(BaseModel):
+    ti_id: UUID
+    type: Literal["ValidateInletsAndOutlets"] = "ValidateInletsAndOutlets"
+
+
 class GetPrevSuccessfulDagRun(BaseModel):
     ti_id: UUID
     type: Literal["GetPrevSuccessfulDagRun"] = "GetPrevSuccessfulDagRun"
@@ -616,6 +674,7 @@ ToSupervisor = Annotated[
         GetXCom,
         GetXComCount,
         GetXComSequenceItem,
+        GetXComSequenceSlice,
         PutVariable,
         RescheduleTask,
         RetryTask,
@@ -623,6 +682,7 @@ ToSupervisor = Annotated[
         SetXCom,
         SkipDownstreamTasks,
         SucceedTask,
+        ValidateInletsAndOutlets,
         TaskState,
         TriggerDagRun,
         DeleteVariable,
