@@ -45,12 +45,14 @@ from airflow.sdk.api import client as sdk_client
 from airflow.sdk.api.client import ServerResponseError
 from airflow.sdk.api.datamodels._generated import (
     AssetEventResponse,
+    AssetProfile,
     AssetResponse,
     DagRunState,
     TaskInstance,
     TaskInstanceState,
 )
 from airflow.sdk.exceptions import AirflowRuntimeError, ErrorType
+from airflow.sdk.execution_time import task_runner
 from airflow.sdk.execution_time.comms import (
     AssetEventsResult,
     AssetResult,
@@ -75,6 +77,8 @@ from airflow.sdk.execution_time.comms import (
     GetVariable,
     GetXCom,
     GetXComSequenceItem,
+    GetXComSequenceSlice,
+    InactiveAssetsResult,
     OKResponse,
     PrevSuccessfulDagRunResult,
     PutVariable,
@@ -88,11 +92,21 @@ from airflow.sdk.execution_time.comms import (
     TaskStatesResult,
     TICount,
     TriggerDagRun,
+    ValidateInletsAndOutlets,
     VariableResult,
     XComResult,
+    XComSequenceIndexResult,
+    XComSequenceSliceResult,
+)
+from airflow.sdk.execution_time.supervisor import (
+    BUFFER_SIZE,
+    ActivitySubprocess,
+    InProcessSupervisorComms,
+    InProcessTestSupervisor,
+    mkpipe,
+    set_supervisor_comms,
+    supervise,
 )
-from airflow.sdk.execution_time.secrets_masker import SecretsMasker
-from airflow.sdk.execution_time.supervisor import BUFFER_SIZE, ActivitySubprocess, mkpipe, supervise
 from airflow.sdk.execution_time.task_runner import CommsDecoder
 from airflow.utils import timezone, timezone as tz
 
@@ -950,6 +964,73 @@ class TestWatchedSubprocessKill:
         # Validate that `_check_subprocess_exit` is called
         mock_process.wait.assert_called_once_with(timeout=0)
 
+    def test_max_wait_time_prevents_cpu_spike(self, watched_subprocess, mock_process, monkeypatch):
+        """Test that max_wait_time calculation prevents CPU spike when heartbeat timeout is reached."""
+        # Mock the configuration to reproduce the CPU spike scenario
+        # Set heartbeat timeout to be very small relative to MIN_HEARTBEAT_INTERVAL
+        monkeypatch.setattr("airflow.sdk.execution_time.supervisor.HEARTBEAT_TIMEOUT", 1)
+        monkeypatch.setattr("airflow.sdk.execution_time.supervisor.MIN_HEARTBEAT_INTERVAL", 10)
+
+        # Set up a scenario where the last successful heartbeat was a long time ago
+        # This will cause the heartbeat calculation to result in a negative value
+        mock_process._last_successful_heartbeat = time.monotonic() - 100  # 100 seconds ago
+
+        # Mock process to still be alive (not exited)
+        mock_process.wait.side_effect = psutil.TimeoutExpired(pid=12345, seconds=0)
+
+        # Call _service_subprocess which is used in _monitor_subprocess
+        # This tests the max_wait_time calculation directly
+        watched_subprocess._service_subprocess(max_wait_time=0.005)  # Very small timeout to verify our fix
+
+        # Verify that selector.select was called with a minimum timeout of 0.01
+        # This proves our fix prevents the timeout=0 scenario that causes CPU spike
+        watched_subprocess.selector.select.assert_called_once()
+        call_args = watched_subprocess.selector.select.call_args
+        timeout_arg = call_args[1]["timeout"] if "timeout" in call_args[1] else call_args[0][0]
+
+        # The timeout should be at least 0.01 (our minimum), never 0
+        assert timeout_arg >= 0.01, f"Expected timeout >= 0.01, got {timeout_arg}"
+
+    @pytest.mark.parametrize(
+        ["heartbeat_timeout", "min_interval", "heartbeat_ago", "expected_min_timeout"],
+        [
+            # Normal case: heartbeat is recent, should use calculated value
+            pytest.param(30, 5, 5, 0.01, id="normal_heartbeat"),
+            # Edge case: heartbeat timeout exceeded, should use minimum
+            pytest.param(10, 20, 50, 0.01, id="heartbeat_timeout_exceeded"),
+            # Bug reproduction case: timeout < interval, heartbeat very old
+            pytest.param(5, 10, 100, 0.01, id="cpu_spike_scenario"),
+        ],
+    )
+    def test_max_wait_time_calculation_edge_cases(
+        self,
+        watched_subprocess,
+        mock_process,
+        monkeypatch,
+        heartbeat_timeout,
+        min_interval,
+        heartbeat_ago,
+        expected_min_timeout,
+    ):
+        """Test max_wait_time calculation in various edge case scenarios."""
+        monkeypatch.setattr("airflow.sdk.execution_time.supervisor.HEARTBEAT_TIMEOUT", heartbeat_timeout)
+        monkeypatch.setattr("airflow.sdk.execution_time.supervisor.MIN_HEARTBEAT_INTERVAL", min_interval)
+
+        watched_subprocess._last_successful_heartbeat = time.monotonic() - heartbeat_ago
+        mock_process.wait.side_effect = psutil.TimeoutExpired(pid=12345, seconds=0)
+
+        # Call the method and verify timeout is never less than our minimum
+        watched_subprocess._service_subprocess(
+            max_wait_time=999
+        )  # Large value, should be overridden by calculation
+
+        # Extract the timeout that was actually used
+        watched_subprocess.selector.select.assert_called_once()
+        call_args = watched_subprocess.selector.select.call_args
+        actual_timeout = call_args[1]["timeout"] if "timeout" in call_args[1] else call_args[0][0]
+
+        assert actual_timeout >= expected_min_timeout
+
 
 class TestHandleRequest:
     @pytest.fixture
@@ -968,9 +1049,17 @@ class TestHandleRequest:
 
         return subprocess, read_end
 
-    @patch("airflow.sdk.execution_time.secrets_masker._secrets_masker")
+    @patch("airflow.sdk.execution_time.supervisor.mask_secret")
     @pytest.mark.parametrize(
-        ["message", "expected_buffer", "client_attr_path", "method_arg", "method_kwarg", "mock_response"],
+        [
+            "message",
+            "expected_buffer",
+            "client_attr_path",
+            "method_arg",
+            "method_kwarg",
+            "mock_response",
+            "mask_secret_args",
+        ],
         [
             pytest.param(
                 GetConnection(conn_id="test_conn"),
@@ -979,8 +1068,19 @@ class TestHandleRequest:
                 ("test_conn",),
                 {},
                 ConnectionResult(conn_id="test_conn", conn_type="mysql"),
+                None,
                 id="get_connection",
             ),
+            pytest.param(
+                GetConnection(conn_id="test_conn"),
+                b'{"conn_id":"test_conn","conn_type":"mysql","password":"password","type":"ConnectionResult"}\n',
+                "connections.get",
+                ("test_conn",),
+                {},
+                ConnectionResult(conn_id="test_conn", conn_type="mysql", password="password"),
+                ["password"],
+                id="get_connection_with_password",
+            ),
             pytest.param(
                 GetConnection(conn_id="test_conn"),
                 b'{"conn_id":"test_conn","conn_type":"mysql","schema":"mysql","type":"ConnectionResult"}\n',
@@ -988,6 +1088,7 @@ class TestHandleRequest:
                 ("test_conn",),
                 {},
                 ConnectionResult(conn_id="test_conn", conn_type="mysql", schema="mysql"),  # type: ignore[call-arg]
+                None,
                 id="get_connection_with_alias",
             ),
             pytest.param(
@@ -997,6 +1098,7 @@ class TestHandleRequest:
                 ("test_key",),
                 {},
                 VariableResult(key="test_key", value="test_value"),
+                ["test_value", "test_key"],
                 id="get_variable",
             ),
             pytest.param(
@@ -1006,6 +1108,7 @@ class TestHandleRequest:
                 ("test_key", "test_value", "test_description"),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="set_variable",
             ),
             pytest.param(
@@ -1015,6 +1118,7 @@ class TestHandleRequest:
                 ("test_key",),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="delete_variable",
             ),
             pytest.param(
@@ -1024,6 +1128,7 @@ class TestHandleRequest:
                 (TI_ID, DeferTask(next_method="execute_callback", classpath="my-classpath")),
                 {},
                 "",
+                None,
                 id="patch_task_instance_to_deferred",
             ),
             pytest.param(
@@ -1042,6 +1147,7 @@ class TestHandleRequest:
                 ),
                 {},
                 "",
+                None,
                 id="patch_task_instance_to_up_for_reschedule",
             ),
             pytest.param(
@@ -1051,6 +1157,7 @@ class TestHandleRequest:
                 ("test_dag", "test_run", "test_task", "test_key", None, False),
                 {},
                 XComResult(key="test_key", value="test_value"),
+                None,
                 id="get_xcom",
             ),
             pytest.param(
@@ -1062,6 +1169,7 @@ class TestHandleRequest:
                 ("test_dag", "test_run", "test_task", "test_key", 2, False),
                 {},
                 XComResult(key="test_key", value="test_value"),
+                None,
                 id="get_xcom_map_index",
             ),
             pytest.param(
@@ -1071,6 +1179,7 @@ class TestHandleRequest:
                 ("test_dag", "test_run", "test_task", "test_key", None, False),
                 {},
                 XComResult(key="test_key", value=None, type="XComResult"),
+                None,
                 id="get_xcom_not_found",
             ),
             pytest.param(
@@ -1086,6 +1195,7 @@ class TestHandleRequest:
                 ("test_dag", "test_run", "test_task", "test_key", None, True),
                 {},
                 XComResult(key="test_key", value=None, type="XComResult"),
+                None,
                 id="get_xcom_include_prior_dates",
             ),
             pytest.param(
@@ -1109,6 +1219,7 @@ class TestHandleRequest:
                 ),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="set_xcom",
             ),
             pytest.param(
@@ -1133,6 +1244,7 @@ class TestHandleRequest:
                 ),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="set_xcom_with_map_index",
             ),
             pytest.param(
@@ -1158,6 +1270,7 @@ class TestHandleRequest:
                 ),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="set_xcom_with_map_index_and_mapped_length",
             ),
             pytest.param(
@@ -1179,6 +1292,7 @@ class TestHandleRequest:
                 ),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="delete_xcom",
             ),
             # we aren't adding all states under TaskInstanceState here, because this test's scope is only to check
@@ -1190,6 +1304,7 @@ class TestHandleRequest:
                 (),
                 {},
                 "",
+                None,
                 id="patch_task_instance_to_skipped",
             ),
             pytest.param(
@@ -1205,6 +1320,7 @@ class TestHandleRequest:
                     "rendered_map_index": "test retry task",
                 },
                 "",
+                None,
                 id="up_for_retry",
             ),
             pytest.param(
@@ -1214,6 +1330,7 @@ class TestHandleRequest:
                 (TI_ID, {"field1": "rendered_value1", "field2": "rendered_value2"}),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="set_rtif",
             ),
             pytest.param(
@@ -1223,6 +1340,7 @@ class TestHandleRequest:
                 [],
                 {"name": "asset"},
                 AssetResult(name="asset", uri="s3://bucket/obj", group="asset"),
+                None,
                 id="get_asset_by_name",
             ),
             pytest.param(
@@ -1232,6 +1350,7 @@ class TestHandleRequest:
                 [],
                 {"uri": "s3://bucket/obj"},
                 AssetResult(name="asset", uri="s3://bucket/obj", group="asset"),
+                None,
                 id="get_asset_by_uri",
             ),
             pytest.param(
@@ -1254,6 +1373,7 @@ class TestHandleRequest:
                         )
                     ]
                 ),
+                None,
                 id="get_asset_events_by_uri_and_name",
             ),
             pytest.param(
@@ -1276,6 +1396,7 @@ class TestHandleRequest:
                         )
                     ]
                 ),
+                None,
                 id="get_asset_events_by_uri",
             ),
             pytest.param(
@@ -1298,6 +1419,7 @@ class TestHandleRequest:
                         )
                     ]
                 ),
+                None,
                 id="get_asset_events_by_name",
             ),
             pytest.param(
@@ -1320,8 +1442,21 @@ class TestHandleRequest:
                         )
                     ]
                 ),
+                None,
                 id="get_asset_events_by_asset_alias",
             ),
+            pytest.param(
+                ValidateInletsAndOutlets(ti_id=TI_ID),
+                b'{"inactive_assets":[{"name":"asset_name","uri":"asset_uri","type":"asset"}],"type":"InactiveAssetsResult"}\n',
+                "task_instances.validate_inlets_and_outlets",
+                (TI_ID,),
+                {},
+                InactiveAssetsResult(
+                    inactive_assets=[AssetProfile(name="asset_name", uri="asset_uri", type="asset")]
+                ),
+                None,
+                id="validate_inlets_and_outlets",
+            ),
             pytest.param(
                 SucceedTask(
                     end_date=timezone.parse("2024-10-31T12:00:00Z"), rendered_map_index="test success task"
@@ -1337,6 +1472,7 @@ class TestHandleRequest:
                     "rendered_map_index": "test success task",
                 },
                 "",
+                None,
                 id="succeed_task",
             ),
             pytest.param(
@@ -1355,6 +1491,7 @@ class TestHandleRequest:
                     data_interval_start=timezone.parse("2025-01-10T12:00:00Z"),
                     data_interval_end=timezone.parse("2025-01-10T14:00:00Z"),
                 ),
+                None,
                 id="get_prev_successful_dagrun",
             ),
             pytest.param(
@@ -1370,6 +1507,7 @@ class TestHandleRequest:
                 ("test_dag", "test_run", {"key": "value"}, timezone.datetime(2025, 1, 1), True),
                 {},
                 OKResponse(ok=True),
+                None,
                 id="dag_run_trigger",
             ),
             pytest.param(
@@ -1379,6 +1517,7 @@ class TestHandleRequest:
                 ("test_dag", "test_run", None, None, False),
                 {},
                 ErrorResponse(error=ErrorType.DAGRUN_ALREADY_EXISTS),
+                None,
                 id="dag_run_trigger_already_exists",
             ),
             pytest.param(
@@ -1388,6 +1527,7 @@ class TestHandleRequest:
                 ("test_dag", "test_run"),
                 {},
                 DagRunStateResult(state=DagRunState.RUNNING),
+                None,
                 id="get_dag_run_state",
             ),
             pytest.param(
@@ -1397,6 +1537,7 @@ class TestHandleRequest:
                 (TI_ID, 1),
                 {},
                 TaskRescheduleStartDate(start_date=timezone.parse("2024-10-31T12:00:00Z")),
+                None,
                 id="get_task_reschedule_start_date",
             ),
             pytest.param(
@@ -1414,6 +1555,7 @@ class TestHandleRequest:
                     "task_ids": ["task1", "task2"],
                 },
                 TICount(count=2),
+                None,
                 id="get_ti_count",
             ),
             pytest.param(
@@ -1428,6 +1570,7 @@ class TestHandleRequest:
                     "states": ["success", "failed"],
                 },
                 DRCount(count=2),
+                None,
                 id="get_dr_count",
             ),
             pytest.param(
@@ -1444,6 +1587,7 @@ class TestHandleRequest:
                     "task_group_id": "test_group",
                 },
                 TaskStatesResult(task_states={"run_id": {"task1": "success", "task2": "failed"}}),
+                None,
                 id="get_task_states",
             ),
             pytest.param(
@@ -1454,11 +1598,12 @@ class TestHandleRequest:
                     task_id="test_task",
                     offset=0,
                 ),
-                b'{"key":"test_key","value":"test_value","type":"XComResult"}\n',
+                b'{"root":"test_value","type":"XComSequenceIndexResult"}\n',
                 "xcoms.get_sequence_item",
                 ("test_dag", "test_run", "test_task", "test_key", 0),
                 {},
-                XComResult(key="test_key", value="test_value"),
+                XComSequenceIndexResult(root="test_value"),
+                None,
                 id="get_xcom_seq_item",
             ),
             pytest.param(
@@ -1474,13 +1619,32 @@ class TestHandleRequest:
                 ("test_dag", "test_run", "test_task", "test_key", 2),
                 {},
                 ErrorResponse(error=ErrorType.XCOM_NOT_FOUND),
+                None,
                 id="get_xcom_seq_item_not_found",
             ),
+            pytest.param(
+                GetXComSequenceSlice(
+                    key="test_key",
+                    dag_id="test_dag",
+                    run_id="test_run",
+                    task_id="test_task",
+                    start=None,
+                    stop=None,
+                    step=None,
+                ),
+                b'{"root":["foo","bar"],"type":"XComSequenceSliceResult"}\n',
+                "xcoms.get_sequence_slice",
+                ("test_dag", "test_run", "test_task", "test_key", None, None, None),
+                {},
+                XComSequenceSliceResult(root=["foo", "bar"]),
+                None,
+                id="get_xcom_seq_slice",
+            ),
         ],
     )
     def test_handle_requests(
         self,
-        mock_secrets_masker,
+        mock_mask_secret,
         watched_subprocess,
         mocker,
         time_machine,
@@ -1490,6 +1654,7 @@ class TestHandleRequest:
         method_arg,
         method_kwarg,
         mock_response,
+        mask_secret_args,
     ):
         """
         Test handling of different messages to the subprocess. For any new message type, add a
@@ -1502,7 +1667,6 @@ class TestHandleRequest:
             3. Checks that the buffer is updated with the expected response.
             4. Verifies that the response is correctly decoded.
         """
-        mock_secrets_masker.return_value = SecretsMasker()
         watched_subprocess, read_socket = watched_subprocess
 
         # Mock the client method. E.g. `client.variables.get` or `client.connections.get`
@@ -1515,6 +1679,10 @@ class TestHandleRequest:
         next(generator)
         msg = message.model_dump_json().encode() + b"\n"
         generator.send(msg)
+
+        if mask_secret_args:
+            mock_mask_secret.assert_called_with(*mask_secret_args)
+
         time_machine.move_to(timezone.datetime(2024, 10, 31), tick=False)
 
         # Verify the correct client method was called
@@ -1600,3 +1768,84 @@ class TestHandleRequest:
             "message": str(error),
             "detail": error.response.json(),
         }
+
+
+class TestSetSupervisorComms:
+    class DummyComms:
+        pass
+
+    @pytest.fixture(autouse=True)
+    def cleanup_supervisor_comms(self):
+        # Ensure clean state before/after test
+        if hasattr(task_runner, "SUPERVISOR_COMMS"):
+            delattr(task_runner, "SUPERVISOR_COMMS")
+        yield
+        if hasattr(task_runner, "SUPERVISOR_COMMS"):
+            delattr(task_runner, "SUPERVISOR_COMMS")
+
+    def test_set_supervisor_comms_overrides_and_restores(self):
+        task_runner.SUPERVISOR_COMMS = self.DummyComms()
+        original = task_runner.SUPERVISOR_COMMS
+        replacement = self.DummyComms()
+
+        with set_supervisor_comms(replacement):
+            assert task_runner.SUPERVISOR_COMMS is replacement
+        assert task_runner.SUPERVISOR_COMMS is original
+
+    def test_set_supervisor_comms_sets_temporarily_when_not_set(self):
+        assert not hasattr(task_runner, "SUPERVISOR_COMMS")
+        replacement = self.DummyComms()
+
+        with set_supervisor_comms(replacement):
+            assert task_runner.SUPERVISOR_COMMS is replacement
+        assert not hasattr(task_runner, "SUPERVISOR_COMMS")
+
+    def test_set_supervisor_comms_unsets_temporarily_when_not_set(self):
+        assert not hasattr(task_runner, "SUPERVISOR_COMMS")
+
+        # This will delete an attribute that isn't set, and restore it likewise
+        with set_supervisor_comms(None):
+            assert not hasattr(task_runner, "SUPERVISOR_COMMS")
+
+        assert not hasattr(task_runner, "SUPERVISOR_COMMS")
+
+
+class TestInProcessTestSupervisor:
+    def test_inprocess_supervisor_comms_roundtrip(self):
+        """
+        Test that InProcessSupervisorComms correctly sends a message to the supervisor,
+        and that the supervisor's response is received via the message queue.
+
+        This verifies the end-to-end communication flow:
+        - send_request() dispatches a message to the supervisor
+        - the supervisor handles the request and appends a response via send_msg()
+        - get_message() returns the enqueued response
+
+        This test mocks the supervisor's `_handle_request()` method to simulate
+        a simple echo-style response, avoiding full task execution.
+        """
+
+        class MinimalSupervisor(InProcessTestSupervisor):
+            def _handle_request(self, msg, log):
+                resp = VariableResult(key=msg.key, value="value")
+                self.send_msg(resp)
+
+        supervisor = MinimalSupervisor(
+            id="test",
+            pid=123,
+            requests_fd=-1,
+            process=MagicMock(),
+            process_log=MagicMock(),
+            client=MagicMock(),
+        )
+        comms = InProcessSupervisorComms(supervisor=supervisor)
+        supervisor.comms = comms
+
+        test_msg = GetVariable(key="test_key")
+
+        comms.send_request(log=MagicMock(), msg=test_msg)
+
+        # Ensure we got back what we expect
+        response = comms.get_message()
+        assert isinstance(response, VariableResult)
+        assert response.value == "value"
